/**********************************************************
　　製作者　　アドウィン
　　解　説　　タイマ割り込みの追加 – プログラム例
**********************************************************/
#include <Arduino.h>      // Arduino.h の読み込み
#include "char8x8.h"      // char8x8.h の読み込み
#include "dotmatrixled.h" // dotmatrixled.h の読み込み
#include "switch.h"       // switch.h の読み込み

// ステッピングモータの動作ピン
int smPins[4] = {2, 5, 16, 17};

// スピーカ用ピンの設定
int speaker = 25;
// スピーカ用PWMチャンネルの設定
int speakerPwm = 0;

// ドットマトリクスLEDを点灯させる行
int ledRow = 0;

// SW1の割り込み時に実行する関数
void intSw1(void) {
  ledcWrite(speakerPwm, 65536 / 2); // スピーカから音を鳴らす
}

// SW2の割り込み時に実行する関数
void intSw2(void) {
  ledcWrite(speakerPwm, 0); // スピーカの音を止める
}

// ステッピングモータ用のタイマ割り込み時に実行する関数
void intSM(void) {

  // ステッピングモータの位相をスタティック変数で宣言
  static int i = 0;

  // 各動作ピンの出力設定
  for (int j = 0; j < 4; j++) {
    if (i == j) { // i番目のピンをHレベルに設定
      digitalWrite(smPins[j], HIGH);
    } else { // i番目以外のピンをLレベルに設定
      digitalWrite(smPins[j], LOW);
    }
  }

  // ステッピングモータの位相をひとつ進める
  i++;
  if (i >= 4) i = 0;
}

// ドットマトリクス用のタイマ割り込み時に実行する関数
void intLED(void) {

  // ドットマトリクスLEDの表示行をひとつ進める
  ledRow++;
  if (ledRow >= 8) ledRow = 0;
}

/*
 * setup 関数
 */
void setup() {
  /* ここにプログラムの最初に1回だけ実行したい処理を上から順に記述していきます */

  // シリアル通信を115200bpsで開始
  Serial.begin(115200);

  Serial.println("初期設定の開始");

  // ドットマトリクスLED初期設定
  initLed();

  // スイッチの初期設定
  initSwitch();

  // ステッピングモータの動作ピンの初期設定
  for (int i = 0; i < 4; i++) {

    // 出力ピンに設定
    pinMode(smPins[i], OUTPUT);
    // Lレベルに設定
    digitalWrite(smPins[i], LOW);
  }

  // スピーカのPWMの設定
  // PWMピンを出力ピンに設定
  pinMode(speaker, OUTPUT);
  // PWMチャンネルを440Hz、16ビット（0〜65536）に設定
  ledcSetup(speakerPwm, 440, 16);
  // PWMピンとPWMチャンネルを結びつける
  ledcAttachPin(speaker, speakerPwm);
  // PWMのデューティ比を0%に設定（消音）
  ledcWrite(speakerPwm, 0);

  // SW1の割り込みの設定
  attachInterrupt(sw1, intSw1, FALLING);
  // SW2の割り込みの設定
  attachInterrupt(sw2, intSw2, FALLING);

  // ステッピングモータ用のタイマ割り込み設定
  // タイマ2を分周80で使用（1MHz）
  hw_timer_t *tmSM = timerBegin(2, 80, true);
  // 割り込み時に実行する関数を設定
  timerAttachInterrupt(tmSM, intSM, false);
  // 割り込みを1秒間隔で周期的に発生させるよう設定
  timerAlarmWrite(tmSM, 1000000, true);
  // タイマ割り込みの有効化
  timerAlarmEnable(tmSM);

  // ドットマトリクスLED用のタイマ割り込み設定
  // タイマ3を分周80で使用（1MHz）
  hw_timer_t *tmLED = timerBegin(3, 80, true);
  // 割り込み時に実行する関数を設定
  timerAttachInterrupt(tmLED, intLED, false);
  // 割り込みを1ms間隔で周期的に発生させるよう設定
  timerAlarmWrite(tmLED, 1000, true);
  // タイマ割り込みの有効化
  timerAlarmEnable(tmLED);

  Serial.println("初期設定の完了");
}

/*
 * loop 関数
 */
void loop() {
  /* ここにプログラム中で繰り返し実行したい処理を上から順に記述していきます */

  // ドットマトリクスLEDの表示の更新
  setLed(ledPatternA[ledRow], ledCathodePattern[ledRow]);

  delay(1); // 時間待ち
}